#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  
#  Copyright 2017-2018 krasytod <krasytod@gmail.com>
###
#  Copyright 2018 krasytod <krasytod@krasytod-desktop>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  
import psycopg2,requests,sql_signatures
import psycopg2.extras
import datetime,time,calendar
from decimal import *
import configparser,argparse,traceback
import logging,os,sys
import signal
from lockfile.pidlockfile import PIDLockFile
from lockfile import AlreadyLocked
from logging.handlers import TimedRotatingFileHandler
from logging.handlers import RotatingFileHandler
from get_config import AUTOINVOICE as AUTOINVOICE_config
from get_config import port as server_port
from get_config import database
PIDF = 'pid_auto_tax.pid'

from get_config import (API_HOST, AUTH_HOST, DEFAULT_AUTH_TOKEN)


class GracefulKiller:
	kill_now = False
	def __init__(self,logger):
		signal.signal(signal.SIGINT, self.exit_gracefully)
		signal.signal(signal.SIGTERM, self.exit_gracefully)
		self.logger= logger

	def exit_gracefully(self,signum, frame):
		pidfile = PIDLockFile(PIDF, timeout=-1)
		try:
			pidfile.break_lock()
		except Exception as e:
			msg = "Exception when trying to break lock %s" %e
			print (msg)
			logger.error(msg)
			try:
				os.remove(PIDF)
				exit(1)
			except Exception as e:
				msg = "Exception when trying to break lock and delete pid file %s" %e
				print (msg)
				logger.error(msg)
				exit(2)
		exit(0)


def connect_to_postgresql(host, port, database, user, password=None):
	try:
		conn = psycopg2.connect(host=host, port=port, database=database, user=user, password=password)
		conn.autocommit = True
		cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
		print("connected to DB")
	except Exception as e: 
		print (" Exception in DB connecting:  %s" % (str(e) ) )
		raise 'I am unable to connect to the PostgreSQL. %s'% (str(e))
	return conn, cursor



def pidCheck(logger):
	pidfile = PIDLockFile(PIDF, timeout=-1)
	try:
		pidfile.acquire()
		msg = "Lock acquered! Daemon setup script ID %s" %pidfile.read_pid()
		print(msg)
		logger.info(msg)
		#pidfile.break_lock()
	except AlreadyLocked:
		msg = "AlreadyLocked! Please kill the current running instance and try again"
		print(msg)
		logger.info(msg)
		exit(1)
		
	except Exception as e:   #No process with locked PID
		msg = "General Exception when trying to get lock %s" %e
		print(msg)
		logger.info(msg)
		exit(1)

def postgres(logger):
	try:
		pg_conn, pg_cur = connect_to_postgresql(database["host"],int(database ["port"]), database ["database"], database ["user"],database ["password"]) #config ["password"]
		
		return pg_conn, pg_cur
	except Exception as e: 
		print (" Exception in connect_to_postgresql:  %s" % (str(e) ) )
		logger.error(" Exception in connect_to_postgresql:  %s" % (str(e) ) )
		return -1,-1

def _get_token():
	resp = requests.post(AUTH_HOST)
	return resp.json()["token"]

#traceback.print_exc(file=sys.stdout)

def get_cost_by_client(pg_cur, invoice_dates, egress_ids, auth_token):
	if egress_ids:
		headers = {
			"Authorization": "Token {}".format(auth_token)
		}
		params = {
			"start_time": "-3600",  #invoice_dates[0],
			#"end_time": invoice_dates[1],
			"step": "all",
			"method": "total",
			"field": "calls",					  #"calls",
			"egress_id": egress_ids,
			"group": "source_number",
			#"output": "http"
		}
		print(params)
		timeout = (1, 5)
		#LOG.debug("all_area_codes_summary_report")
		cost= 0.0
		calls = requests.get(
			API_HOST, params=params, headers=headers, timeout=timeout)
		print("CALLS  %s" % str( calls.json() ) )
		if calls.json()["data"]:
			#print ("their is calls")
			params["field"] = "egress_billed_time"
			egress_billed_time = requests.get(
				API_HOST, params=params, headers=headers, timeout=timeout)
			print("egress_billed_time %s"%str(egress_billed_time.json()))
			params["field"] = "egress_cost"
			egress_cost = requests.get(
				API_HOST, params=params, headers=headers, timeout=timeout)
			print("egress_cost %s"%str(egress_cost.json()))
			#d_calls = [item for item in calls.json()["data"] if item]
			#d_time = [item for item in egress_billed_time.json()["data"] if item]
			d_cost = [item for item in egress_cost.json()["data"] if item]
			print(d_cost)
			
			for row in d_cost:
				egress_cost = row.get('egress_cost',0.0)
				try:
					cost += float(egress_cost)
				except Exception as e :
					print ("Exception in code line 158. %s" %e )
				
	print ("Cost",cost)
	return cost


def get_clients(pg_cur,logger):
	"""We need active clients for DID"""
	sql = """select client_id,name from client where status  is true and client_type = 1 and mode  =2 and include_tax is true;"""
	msg = "Query for getting clients that must be taxed %s" % sql
	print(msg)
	logger.debug(msg)
	pg_cur.execute(sql)
	clients = pg_cur.fetchall()
	return clients

def main_start(pg_cur,logger):
	
	#first, get active postpaid clients for DID
	clients= get_clients(pg_cur,logger)
	#autenticate to the API
	auth_token = _get_token()
	sql_balance = """INSERT INTO client_balance_operation_action  (client_id, balance,ingress_balance ,update_result,action ,create_by)   VALUES  
		({client_id},{fee},{fee},0,3,'DID Invoice'  )  """
	now =  datetime.datetime.now()
	start_time = now - datetime.timedelta(minutes=65)
	end_time = now - datetime.timedelta(minutes=5)
	time_frame = [ int( start_time.timestamp() ) ,int( end_time.timestamp() ) ] 
	for client in clients:
		msg = "Working on %s with client_id %s" % (client['name'],client['client_id'])
		print (msg)
		logger.debug(msg)
		#get egress
		query = sql_signatures.egress_ids.format(client['client_id'])
		pg_cur.execute(query)
		for row in pg_cur.fetchall():
			egress_ids = row[0]
		egress_cost = get_cost_by_client(
			pg_cur, time_frame, egress_ids, auth_token)
		#get tax
		taxes_list=[] 
		whole_tax = 0.0
		query = sql_signatures.include_tax.format(client['client_id'])
		pg_cur.execute(query)
		for row in pg_cur.fetchall():
			taxes_list.append( [row[0],row[1]])
		msg = "Taxes for client %s with client_id %s:   %s" %(client['name'],client['client_id'], str(  taxes_list )    )
		for  tax_row in  taxes_list:
			tax_percent_num = tax_row [ 1 ]  #percent value of the tax
			tax =  egress_cost  * (float(tax_percent_num)/100)
			whole_tax += tax
		
		#removing tax from client balance
		client_sql_balance = sql_balance.format(client_id = client['client_id'], fee =whole_tax    )
		if whole_tax > 0.01:
			print("removing tax from client balance", client_sql_balance)
			pg_cur.execute(client_sql_balance)
		else:
			print("Nothing for deduction. whole_tax:",whole_tax)

if __name__ == '__main__':
	logger = logging.getLogger('DID Auto tax Dmon')
	handler = RotatingFileHandler("logs/tax_dmon.log",maxBytes=1024*1024*128 ,backupCount= 10 )
	logger.setLevel(logging.DEBUG)
	
	formatstr = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
	formatstr = '%(asctime)s: %(levelname)s:  %(message)s'
	formatter = logging.Formatter(formatstr)
	handler.setFormatter(formatter)
	logger.addHandler(handler)
	
	
	parser = argparse.ArgumentParser(description='Invoice DID Tax API')
	#parser.add_argument('-a', '--action',   help='Action to be taken. start , stop')  #default='start'
	parser.add_argument('-M', '--minutes', default='60' ,  help='Run every N minutes. Default is 60 minutes')
	args = parser.parse_args()
	try:
		minutes = int(args.minutes)
	except Exception as e:
		print (e)
		minutes = 60
	logger.debug("Starting Did Tax "  ) 
	pidCheck(logger)

	killer = GracefulKiller(logger)
	
	while True:
		pg_conn,pg_cur = postgres(logger)
		if pg_cur == -1:
			info =  " Autoinvoice module start failer. Please check DB settings! "
			logger_dmon.error(info)
			exit (1)
		try:
			main_start(pg_cur,logger)
		except Exception as e:
			msg = "Exception in main module %s" %e
			print (msg)
			logger.error(msg)
			#pidfile = PIDLockFile(PIDF, timeout=-1)
			#pidfile.break_lock()
			#exit(5)
			time.sleep(60)
			pg_cur.close()
			pg_conn.close()
			continue
		pg_cur.close()
		pg_conn.close()
		for dummy in range(minutes*30):
			time.sleep(2) 


