from reportlab.lib.pagesizes import letter
from reportlab.lib import colors
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.lib.units import inch, mm
from reportlab.platypus.flowables import KeepTogether 
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, Image, Table, \
    Frame, BaseDocTemplate, PageTemplate, FrameBreak,PageBreak
import html
from html.parser import HTMLParser
from get_config import static


class ColorFrame(Frame):
    def __init__(
            self, x1, y1, width, height, leftPadding=6, bottomPadding=6,
            rightPadding=6, topPadding=6, id=None, showBoundary=0,
            overlapAttachedSpace=None, _debug=None, background=None):

        Frame.__init__(
            self, x1, y1, width, height, leftPadding,
            bottomPadding, rightPadding, topPadding, id, showBoundary,
            overlapAttachedSpace, _debug)

        self.background = background

    def drawBackground(self, canv):
        color = colors.toColor(self.background)

        canv.saveState()
        canv.setFillColor(color)
        canv.rect(
            self._x1, self._y1, self._x2 - self._x1, self._y2 - self._y1,
            stroke=0, fill=1
        )
        canv.restoreState()

    def addFromList(self, drawlist, canv):
        if self.background:
            self.drawBackground(canv)
        Frame.addFromList(self, drawlist, canv)


class Invoice(object):
    def __init__(self, data, filename):
        self._cont = data
        self._filename = filename
        self._logo = '{}/logo.png'.format(static)
        self._doc = None
        self._story = []
        self._frame = None

        self._style_sheet = getSampleStyleSheet()
        self._style = self._style_sheet["BodyText"]
        self.styleN = self._style_sheet["BodyText"]
        self.styleN.fontSize = 9
        self.styleN.name = "InfoText"
        self._style_sheet.add(self.styleN)
        self._t_style = ParagraphStyle(
            name='Tables', leftIndent=40, rightIndent=40)

    def _add_page_number(self, canvas, doc):
        page_num = canvas.getPageNumber()
        text = "Page #%s" % page_num
        canvas.drawRightString(200*mm, 20*mm, text)
        # self._frame.drawBackground(canvas)

    @staticmethod
    def _get_total_by_column(table, column, column_type=float, round_digits=2):
        return round(
            sum([column_type(row[column]) for row in table]), round_digits)

    def _para(self, content):
        #adding check in case the content is None
        if content == None:
          return Paragraph(" ", self._style)
          
        return Paragraph(content, self._style)

    def _t_para(self, content):
        #adding check in case the content is None
        if content == None:
          return Paragraph(" ", self._t_style)
        return Paragraph(content, self._t_style)

    def _html_para(self,html_src):
        elements = list()
        styleSheet=self._style_sheet
        class Handler(HTMLParser):
            mode = ""
            buffer = ""
            listcounter = 0
            listtype = ""
            color = ""

            def handle_starttag(self, name, attrs):
                if name in ["strong", "em", "i", "b"]:
                    self.mode = name
                elif name == "ol":
                    self.listcounter = 1
                    self.listtype = "ol"
                elif name == "ul":
                    self.listtype = "ul"
                elif name == "hr":
                    elements.append(PageBreak())
                elif name == "span":
                    self.mode = name
                    attrs = dict(attrs)
                    if 'style' in attrs:
                        style = attrs['style'].split(':')
                        if len(style) == 2:
                            self.color = style[1]

            def handle_endtag(self, name):
                if name.startswith("h") and name[-1] in ["1", "2", "3", "4", "5", "6"]:
                    elements.append(Paragraph(self.buffer, styleSheet["Heading%s" % name[-1]]))
                elif name in ["strong", "em", "i", "b", "span"]:
                    self.mode = ""
                elif name == "p":
                    # elements.append(Paragraph(self.buffer+"<br/>", styleSheet["BodyText"]))
                    elements.append(Paragraph(self.buffer, styleSheet["InfoText"]))
                elif name == "li":
                    if self.listtype == "ul":
                        elements.append(Paragraph(self.buffer, styleSheet["InfoText"], bulletText=u"•"))
                    else:
                        elements.append(
                            Paragraph(self.buffer, styleSheet["InfoText"], bulletText="%s." % self.listcounter))
                        self.listcounter += 1
                elif name in ["ol", "ul"]:
                    self.listcounter = 0
                    self.listtype = ""

                if name in ["h1", "h2", "h3", "h4", "h5", "h6", "p", "li"]:
                    self.buffer = ""

            def handle_data(self, chars):
                surrounding = None
                if self.mode in ["span"]:
                    if self.color:
                        surrounding = "font color={}".format(self.color)
                elif self.mode in ["strong", "em", "i", "b"]:
                    if self.mode in ["strong", "b"]:
                        surrounding = "b"
                    else:
                        surrounding = "i"

                if surrounding:
                    chars = "<%s>%s</%s>" % (surrounding, chars, surrounding)

                self.buffer += chars

        if '&lt;' in html_src:
            html_src = html.unescape(html_src)
        try:
            parseString = Handler()
            parseString.feed('<html>{}</html>'.format(html_src))
            parseString.close()
        except:
            elements = [Paragraph(html_src, styleSheet["InfoText"])]

        return elements

    def _para_bank_info(self):
        if self._cont["pdf_tpl"] == None:
          return self._para(" ")
        return self._html_para(self._cont["pdf_tpl"])

    def _render_footer(self):
        data = [
            [""],
            [
                self._para_bank_info() if self._cont["tpl_number"]  in (0,2) else ""
            ],
            [""],
        ]
        return Table(
            data,
            style=[
                ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                ('TOPPADDING', (0, 0), (-1, -1), 0),
                ('ALIGN', (0, 0), (-1, -1), 'CENTER' if self._cont["tpl_number"] == 2 else 'LEFT'),
                ('BOTTOMPADDING', (0, 0), (-1, -1), 0),
            ])

    def _render_header(self):
        logo = Image(self._logo)
        logo.drawHeight = 0.5 * inch
        logo.drawWidth = 1 * inch

        c_name = self._para(
            "<para align=right><font size=9><b>Billed to:</b> {}</font></para>".format(
                self._cont["client_name"]))
        inv_number = self._para(
            "<para align=right><font size=9><b>Invoice#:</b> {}</font></para>".format(
                self._cont["invoice_name"]))
        inv_date = self._para(
            "<para align=right><font size=9><b>Invoice Date:</b> {}</font></para>".format(
                self._cont["bill_date"]))
        due_date = self._para(
            "<para align=right><font size=9><b>Payment Due Date:</b> {}</font></para>".format(
                self._cont["due_date"]))
        billing_period = self._para(
            "<para align=right><font size=9><b>Billing Period:</b> {}</font></para>".format(
                self._cont["billing_period"]))
        client_data = [
            [c_name],
            [inv_number],
            [inv_date],
            [due_date],
            [billing_period],
        ]
        client_table = Table(
            client_data,
            style=[
                ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                ('TOPPADDING', (0, 0), (-1, -1), 0),
                ('BOTTOMPADDING', (0, 0), (-1, -1), 0),
            ])

        p_name = self._html_para(self._cont["company_info"])
        p_bank = self._para_bank_info() if self._cont["tpl_number"] == 1 else ""
        data = [
            [logo, ""],
            ["", ""],
            [p_name, client_table],
            ["", ""],
            [p_bank, ""],
        ]
        return Table(
            data,
            style=[
                ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                ('TOPPADDING', (0, 0), (-1, -1), 0),
                ('BOTTOMPADDING', (0, 0), (-1, -1), 0),
            ])
       # "port_fee_total": round(float(port_fee_total_charges), invoice_decimal_digits),
       # "port_fee_summery": _port_fee_summary_analysis,
    def _render_summary(self):
        account_summary = self._para(
            "<b><font size=11>Account Summary</font></b>")
        prev_balance_label = self._para("<font size=9>Previous Balance</font>")
        prev_balance = self._para(
            "<b><font size=9>{}</font></b>".format(self._cont["prev_balance"]))
        long_distance_charges_label = self._para(
            "<font size=9>Long Distance Charges</font>")
        long_distance_charges = self._para(
            "<b><font size=9>{}</font></b>".format(self._cont["long_distance_charges"]))
        recurring_charges_label = self._para(
            "<font size=9>Recurring Charges</font>")
        recurring_charges = self._para(
            "<b><font size=9>{}</font></b>".format(self._cont["recurring_charges"]))
        
        tax_rows = []
        for tax_row in self._cont["tax"]:
         tax_charges_label = self._para("<font size=9>%s</font>" % tax_row[0]   )
         tax_charges = self._para(
            "<b><font size=9>{}</font></b>".format( tax_row[1] )  )
         tax_rows.append( [ tax_charges_label, tax_charges] )
        tax_rows.append( [ self._para("<font size=9>%s</font>" % "Tax Sum"  ) ,  self._para("<b><font size=9>{}</font></b>".format( self._cont["whole_tax"] )  ) ])
        
        
        port_charges_label = self._para(
            "<font size=9>Port Charge</font>")
        port_charges = self._para(
            "<b><font size=9>{}</font></b>".format(self._cont["port_fee_total"]))
            
        amount_due_label = self._para("<font size=9>Amount Due</font>")
        amount_due = self._para(
            "<b><font size=9>{}</font></b>".format(self._cont["amount_due"]))
        due_date_label = self._para("<font size=9>Due Date</font>")
        due_date = self._para(
            "<b><font size=9>{}</font></b>".format(self._cont["due_date"]))
        if self._cont["include_tax"] ==False:
         data = [
            [account_summary, ""],
            [prev_balance_label, prev_balance],
            [long_distance_charges_label, long_distance_charges],
            [recurring_charges_label, recurring_charges],
            [port_charges_label,port_charges],
            [amount_due_label, amount_due],
            [due_date_label, due_date]
            
         ]
        else:
         data = [
            [account_summary, ""],
            [prev_balance_label, prev_balance],
            [long_distance_charges_label, long_distance_charges],
            [recurring_charges_label, recurring_charges],
            [port_charges_label,port_charges] ]
         data.extend(tax_rows)
         data.extend(            
           [[amount_due_label, amount_due],
            [due_date_label, due_date] ]
            )

        return Table(
            data,
            style=[
                ('BOTTOMPADDING', (0, 0), (1, 0), 4),
                ('TOPPADDING', (0, 1), (-1, -1), 0),
                ('BOTTOMPADDING', (0, 1), (-1, -1), 0),
                ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                ('SPAN', (0, 0), (1, 0)),
                ('LINEABOVE', (0, 1), (1, 1), 0.5, colors.black),
                ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
            ])

    def _render_transaction_summary_analysis(self):
        table_header = self._para(
            "<b><font size=11>Transaction Summary Analysis</font></b>")
        date_label = self._para("<b><font size=9>Date</font></b>")
        description_label = self._para(
            "<b><font size=9>Description</font></b>")
        amount_label = self._para("<b><font size=9>Amount</font></b>")
        data = [
            [table_header, "", ""],
            ["", "", ""],
            [date_label, description_label, amount_label],
        ]
        data.extend(self._cont["transaction_summary_analysis"])
        return Table(
            data,
            style=[
                ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                ('SPAN', (0, 0), (2, 0)),
                ('LINEABOVE', (0, 1), (2, 1), 0.5, colors.black),
                ('BACKGROUND', (0, 2), (2, 2), colors.HexColor('#b5b5b5')),
                ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                ('INNERGRID', (0, 2), (-1, -1), 0.5, colors.white),
                ('ALIGN', (2, 3), (2, -1), 'RIGHT'),
            ])

    @property
    def _total_label(self):
        return self._para("<b><font size=9>TOTAL</font></b>")

    @property
    def _num_calls_label(self):
        return self._para("<b><font size=9># OF CALLS</font></b>")

    @property
    def _total_minutes_label(self):
        return self._para("<b><font size=9>TOTAL MINUTES</font></b>")

    @property
    def _charge_label(self):
        return self._para("<b><font size=9>CHARGE</font></b>")

    @property
    def _location_label(self):
        return self._para("<b><font size=9>LOCATION</font></b>")
        
    @property
    def _per_port_fee_label(self):
        return self._para("<b><font size=9>FEE PER PORT</font></b>")
    
    @property
    def _port_allowed_label(self):
        return self._para("<b><font size=9>PORT ALLOWED</font></b>")
        
    @property
    def _port_charge_label(self):
        return self._para("<b><font size=9>PORT CHARGE</font></b>")
        
    @property
    def _did_label(self):
        return self._para("<b><font size=9>DID</font></b>")


    def _render_authorization_code_summary_report(self):
        report_table = self._cont["authorization_code_summary_report"]
        table_header = self._para(
            "<b><font size=11>AUTHORIZATION CODE (800) SUMMARY REPORT</font></b>")
        authorization_code_label = self._para(
            "<b><font size=9>DID</font></b>")
        name_label = self._para("<b><font size=9>Type</font></b>")
        data = [
            [table_header, "", "", "", ""],
            ["", "", "", "", ""],
            [
                authorization_code_label, name_label, self._num_calls_label,
                self._total_minutes_label, self._charge_label],
        ]
        #report table must be cut without end line
        new_rp_table = []
        for row in report_table:
           new_rp_table.append( row[:-1] ) 
        report_table = new_rp_table
        data.extend(report_table)
        
        total = [
            self._total_label,
            "",
            self._get_total_by_column(report_table, 2, int),
            self._get_total_by_column(report_table, 3, float),
            self._get_total_by_column(report_table, 4, float),
        ]
        data.extend([total])
        return Table(
            data,
            style=[
                ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                ('SPAN', (0, 0), (4, 0)),
                ('LINEABOVE', (0, 1), (4, 1), 0.5, colors.black),
                ('BACKGROUND', (0, 2), (4, 2), colors.HexColor('#b5b5b5')),
                ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                ('INNERGRID', (0, 2), (-1, -1), 0.5, colors.white),
                ('ALIGN', (2, 3), (4, -1), 'RIGHT'),
                ('SPAN', (0, -1), (1, -1)),
                ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')),
            ])


    def _render_all_area_codes_summary_report(self):
        report_table = self._cont["all_area_codes_summary_report"]
        table_header = self._para(
            "<b><font size=11>ALL AREA CODES SUMMARY REPORT</font></b>")
        area_code_inbound_label = self._para(
            "<b><font size=9>AREA CODE INBOUND</font></b>")
        data = [
            [table_header, "", "", "", ""],
            ["", "", "", "", ""],
            [
                area_code_inbound_label, self._location_label,
                self._num_calls_label, self._total_minutes_label,
                self._charge_label],
        ]
        data.extend(report_table)
        total = [
            self._total_label,
            "",
            self._get_total_by_column(report_table, 2, int),
            self._get_total_by_column(report_table, 3, float),
            self._get_total_by_column(report_table, 4, float),
        ]
        data.extend([total])
        return Table(
            data,
            style=[
                ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                ('SPAN', (0, 0), (4, 0)),
                ('LINEABOVE', (0, 1), (4, 1), 0.5, colors.black),
                ('BACKGROUND', (0, 2), (4, 2), colors.HexColor('#b5b5b5')),
                ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                ('INNERGRID', (0, 2), (-1, -1), 0.5, colors.white),
                ('ALIGN', (2, 3), (4, -1), 'RIGHT'),
                ('SPAN', (0, -1), (1, -1)),
                ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')),
            ])


    def _render_port_summary_report(self):
        report_table = self._cont["port fee summary report"]
        #print ("report_table",report_table)
        table_header = self._para(
            "<b><font size=11>PORT FEE SUMMARY REPORT</font></b>")
        port_label = self._para(
            "<b><font size=9>ORIGINATION LATA</font></b>")
        data = [
            [table_header, "", "", ""],
            ["", "", "", ""],
            [
                self._did_label, self._per_port_fee_label,
                self._port_allowed_label, self._port_charge_label],
        ]
        data.extend(report_table)

        total = [
            self._total_label,
            "",
            "",
            self._get_total_by_column(report_table, 3, float),
        ]
        data.extend([total])
        
        return Table(
            data,
            style=[
                ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                ('SPAN', (0, 0), (3, 0)),
                ('LINEABOVE', (0, 1), (3, 1), 0.5, colors.black),
                ('BACKGROUND', (0, 2), (3, 2), colors.HexColor('#b5b5b5')),
                ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                ('INNERGRID', (0, 2), (-1, -1), 0.5, colors.white),
                ('ALIGN', (2, 3), (3, -1), 'RIGHT'),
                ('SPAN', (0, -1), (2, -1)),
                ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')),
            ])
        
        


    def _render_origination_lata_summary_report(self):
        report_table = self._cont["origination_lata_summary_report"]
        table_header = self._para(
            "<b><font size=11>ORIGINATION LATA SUMMARY REPORT</font></b>")
        origination_lata_label = self._para(
            "<b><font size=9>ORIGINATION LATA</font></b>")
        data = [
            [table_header, "", "", "", ""],
            ["", "", "", "", ""],
            [
                origination_lata_label, self._location_label,
                self._num_calls_label, self._total_minutes_label,
                self._charge_label],
        ]
        data.extend(report_table)
        total = [
            self._total_label,
            "",
            self._get_total_by_column(report_table, 2, int),
            self._get_total_by_column(report_table, 3, float),
            self._get_total_by_column(report_table, 4, float),
        ]
        data.extend([total])
        return Table(
            data,
            style=[
                ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                ('SPAN', (0, 0), (4, 0)),
                ('LINEABOVE', (0, 1), (4, 1), 0.5, colors.black),
                ('BACKGROUND', (0, 2), (4, 2), colors.HexColor('#b5b5b5')),
                ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                ('INNERGRID', (0, 2), (-1, -1), 0.5, colors.white),
                ('ALIGN', (2, 3), (4, -1), 'RIGHT'),
                ('SPAN', (0, -1), (1, -1)),
                ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')),
            ])

    def generate(self):
        # self._doc = SimpleDocTemplate(
        #     self._filename, pagesize=letter)
        self._doc = BaseDocTemplate(
            self._filename, pagesize=letter)
        page_width, page_height = letter
        self._frame = Frame(
            self._doc.leftMargin, self._doc.bottomMargin,
            self._doc.width, self._doc.height,
            leftPadding=10, rightPadding=10,
            showBoundary=0)
        self._doc.addPageTemplates(
            [PageTemplate(id='Col', onPage=self._add_page_number, frames=[self._frame]), ])

        self._story.append(self._render_header())
        self._story.append(Spacer(page_width, 30))

        # data = [
        #     [""],
        #     [self._render_summary()],
        #     [self._render_transaction_summary_analysis()],
        #     [self._render_authorization_code_summary_report()],
        #     [self._render_all_area_codes_summary_report()],
        #     [self._render_origination_lata_summary_report()],
        #     [""],
        # ]
        # inv_table = Table(
        #     data,
        #     style=[
        #         ('LEFTPADDING', (0, 0), (-1, -1), 40),
        #         ('RIGHTPADDING', (0, 0), (-1, -1), 40),
        #         ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
        #     ])
        # self._story.append(inv_table)
        if self._cont['params']['account_summary'] == True:
          self._story.append(self._render_summary())
          #account_summary,,,,
        if self._cont['params']['transaction_summary_analysis'] == True:  
           self._story.append(KeepTogether (self._render_transaction_summary_analysis()))
        if self._cont['params']['auth_code_800_summary'] == True:
         self._story.append(KeepTogether (self._render_authorization_code_summary_report()))
        if self._cont['params']['all_area_codes_summary'] == True:
         self._story.append(KeepTogether (self._render_all_area_codes_summary_report()))
        if self._cont['params']['orig_lata_summary'] == True:
         self._story.append(KeepTogether (self._render_origination_lata_summary_report()))
        if self._cont['params'].get('port_summary',False) == True:
         self._story.append(KeepTogether (self._render_port_summary_report()))

        self._story.append(Spacer(page_width, 30))
        self._story.append(self._render_footer())

        self._doc.build(self._story)

        # self._doc.build(
        #     self._story,
        #     onFirstPage=self._add_page_number,
        #     onLaterPages=self._add_page_number)

        return True
