import sys
import logging

import asyncio
from aiohttp import web

import get_config as config
import db
import routes
import argparse ,os ,time
from time import gmtime, strftime
import psycopg2
import psycopg2.extras

VERSION = {'ver':"1.6",'date':'2019-04-18'}

def app_init(loop):
    app = web.Application(loop=loop)
    db.init(app)
    routes.init(app)
    return app

def connect_to_postgresql():
	conn = psycopg2.connect(host=config.database['host'], port=config.database['port'],database= config.database['database'],user= config.database['user'],
	 password=config.database['password'])
	conn.autocommit = True
	cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
	print("connected to DB" )
		
	return conn, cursor




def logVersion():
	pg_con,pg_cur  = connect_to_postgresql()
	#worker_info = VERSION  #
	now_time = time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime())+"+00"
	program_name_dmon = "dnl_did_invoice_server. "
	
	sql = """   SELECT count(*) as count FROM version_information WHERE program_name = '%s'""" % program_name_dmon
	print(sql)
	pg_cur.execute(sql)
	count = pg_cur.fetchone()['count']
	if count == 0:
		sql = """INSERT INTO version_information (program_name ,major_ver,build_date ,start_time ) VALUES (%s,%s,%s,%s)  """
	
	else:
		sql = """UPDATE  version_information  SET  major_ver = '%s' , build_date = '%s',start_time = '%s' WHERE  program_name = '%s'; """ % (VERSION['ver'],VERSION['date'],now_time,program_name_dmon) 
	
	print (sql)
	try:
		pg_cur.execute(sql, (program_name_dmon, VERSION['ver'],VERSION['date'],now_time)  )
		
	
	except Exception as e:  
		print ("Except in versioning %s"%e )
		#logger.error("Except in versioning %s"%e )
		try:
			#stop()
			pass
		except Exception as e:
			pass


def main(argv=sys.argv):

	
	parser = argparse.ArgumentParser(description='Invoice DID API')
	parser.add_argument('-l', '--port', default= 'def' , help='port')  #default='start'
	parser.add_argument('-j', '--host', default= 'def' , help='host')
	now_time = strftime("%Y-%m-%d %H:%M:%S", gmtime())
	parser.add_argument('-d', '--date',  default=now_time , help='The time of script execution')
	cur_path = os.path.dirname(os.path.abspath(__file__))
	parser.add_argument('-p', '--path',  default=str(cur_path ) , help='the path of script ')
	args = parser.parse_args()
	loop = asyncio.get_event_loop()
	app = app_init(loop)
	logVersion()
	
	host = "0.0.0.0"
	port = 18881
	
	try:
		host =  config.host
		port = int(config.port)
		print("Setting port and host from config.ini file  %s %s " %(port,host)    )
	except Exception as e:
		print (e)
	if args.host != 'def':
		print ("Host will be reset from command line to host %s"  % args.host )  
		host = args.host
	if args.port != 'def':
		print ("Host will be reset from command line to port %s"  % args.port )  
		port = int(args.port)
	#host = args.host
	#port = int(args.port)
	print (" Server ver %s is starting on %s:%s"  %  (str(VERSION),host,port))
	
	web.run_app(app, host=host, port=port)


if __name__ == "__main__":
    main()
