import json
import hashlib
import logging
from functools import wraps

from aiohttp.web import Response

from exceptions import ApiError, InternalError


def hash_password(pwd):
    return hashlib.sha1(pwd.encode('utf-8')).hexdigest()


def api_error(exc):
    return api_response({'msg': exc.msg}, exc.status)


def api_response(data, status=200):
    return Response(
            text=json.dumps(data),
            status=status,
            content_type='application/json')


def catch_all(f):
    @wraps(f)
    async def wrapper(*args, **kwargs):
        try:
            return await f(*args, **kwargs)
        except ApiError as exc:
            return api_error(exc)
        except Exception as exc:
            logging.exception(exc)
            return api_error(InternalError())
    return wrapper


def db_ctx(f):
    @wraps(f)
    async def wrapper(request, *args, **kwargs):
        async with request.app['db'].acquire() as connection:
            return await f(request, *args, connection=connection, **kwargs)
    return wrapper
