import http.client as http_client
import logging
import requests

from datetime import datetime
import datetime as dt
import time, traceback,sys

from aiohttp import MultiDict
from aiohttp.web import StreamResponse
from render_invoice import Invoice

from utils import catch_all, api_response, api_error
from get_config import (invoices_path, allowed_hosts, 
					API_HOST, AUTH_HOST, DEFAULT_AUTH_TOKEN, log_format,
					code_deck_id)
import sql_signatures
import os
logging.basicConfig(level=logging.DEBUG, format=log_format)
LOG = logging.getLogger("did_billing." + __name__)

http_client.HTTPConnection.debuglevel = 1
requests_log = logging.getLogger("requests.packages.urllib3")
requests_log.setLevel(logging.DEBUG)
requests_log.propagate = True

invoice_decimal_digits = 2
toll_free = ['800', '888', '877', '866', '855', '844']

RT_API_TIMEOUT = (2,240)

def _all_area_codes_summary_report(conn, invoice_dates, egress_ids, auth_token):
	res = []
	if egress_ids:
		headers = {
			"Authorization": "Token {}".format(auth_token)
		}
		params = {
			"start_time": invoice_dates[0],
			"end_time": invoice_dates[1],
			"step": "86400",
			"method": "total",
			"field": "calls",					  #"calls",
			"egress_id": egress_ids,
			"group": "source_number",
			#"output": "http"
		}
		print(params)
		timeout = RT_API_TIMEOUT
		#LOG.debug("all_area_codes_summary_report")
		calls = requests.get(
			API_HOST, params=params, headers=headers, timeout=timeout)
		LOG.debug("CALLS all_area_codes_summary_report %s" % str( calls.json() )[0:4096] )
		if calls.json()["data"]:
			#print ("their is calls")
			params["field"] = "egress_billed_time"
			egress_billed_time = requests.get(
				API_HOST, params=params, headers=headers, timeout=timeout)
			LOG.debug("egress_billed_time %s"%str(egress_billed_time.json()))
			params["field"] = "egress_cost"
			egress_cost = requests.get(
				API_HOST, params=params, headers=headers, timeout=timeout)
			LOG.debug("egress_cost %s"%str(egress_cost.json()))
			d_calls = [item for item in calls.json()["data"] if item]
			d_time = [item for item in egress_billed_time.json()["data"] if item]
			d_cost = [item for item in egress_cost.json()["data"] if item]
			LOG.debug(d_calls)
			LOG.debug(d_time)
			LOG.debug(d_cost)
			for i, item in enumerate(d_calls):
				# if not is_toll_free(item["destination number"]):
				print (i, item)
				try:
				 res.append((
					
					get_did_area_code(conn, item["source_number"]),
					get_did_location(conn, item["source_number"]),
					#int(item["value"]),
					int(item["calls"]),
					round(float(d_time[i]["egress_billed_time"]) / 60, invoice_decimal_digits),
					round(float(d_cost[i]["egress_cost"]), invoice_decimal_digits)
				))
				except :
				 print ("Exception in code line 82. Possible calls = 0.0 , source_number is omitted from the API")
				 pass
	print ("res ALL AREA CODES SUMMARY REPORT",res)
	return res


def _origination_lata_summary_report(conn, invoice_dates, egress_ids, auth_token):
	res = []
	if egress_ids:
		headers = {
			"Authorization": "Token {}".format(auth_token)
		}
		params = {
			"start_time": invoice_dates[0],
			"end_time": invoice_dates[1],
			"step": "86400",
			"method": "total",
			"field": "calls",
			"egress_id": egress_ids,
			"group": "source_number",
			#"output": "http"
		}
		timeout = RT_API_TIMEOUT
		LOG.debug("Start request _origination_lata_summary_report")
		calls = requests.get(
			API_HOST, params=params, headers=headers, timeout=timeout)
		LOG.debug("CALLS lata_summary %s" % str( calls.json() )[0:4096] )
		if calls.json()["data"]:
			params["field"] = "egress_billed_time"
			egress_billed_time = requests.get(
				API_HOST, params=params, headers=headers, timeout=timeout)
			params["field"] = "egress_cost"
			egress_cost = requests.get(
				API_HOST, params=params, headers=headers, timeout=timeout)
			
			d_calls = [item for item in calls.json()["data"] if item]
			d_time = [item for item in egress_billed_time.json()["data"] if item]
			d_cost = [item for item in egress_cost.json()["data"] if item]
			LOG.debug(d_calls)
			LOG.debug(d_time)
			LOG.debug(d_cost)
			for i, item in enumerate(d_calls):
				# if not is_toll_free(item["destination number"]):
				lata = get_did_lata(conn, item["source_number"])
				print (i,item)
				print ("lata", lata)
				
				location = "Anonymous" if lata == 'OTHERs' else get_did_location(conn, item["source_number"])
				dup = [(j, k) for j, k in enumerate(res) if lata == k[0]]
				if dup:
					index, old_record = dup[0]
					res[index] = (
						lata,
						location,
						old_record[2] + int(item["calls"]),
						round(old_record[3] + float(d_time[i]["egress_billed_time"]) / 60, invoice_decimal_digits),
						round(old_record[4] + float(d_cost[i]["egress_cost"]), invoice_decimal_digits)
					)
				else:
					res.append((
						lata,
						location,
						int(item["calls"]),
						round(float(d_time[i]["egress_billed_time"]) / 60, invoice_decimal_digits),
						round(float(d_cost[i]["egress_cost"]), invoice_decimal_digits)
					))

	return res


def is_toll_free(number):
	if str(number)[0:3] in toll_free:
		return True
	if str(number)[1:4] in toll_free:
		return True
	else:
		return False


def get_did_type(conn, number):
	# US Toll Free
	if str(number)[0:3] in toll_free:
		return 'US Toll Free'
	if str(number)[1:4] in toll_free:
		return 'US Toll Free'
	# US Local
	if str(number)[0] == '1':
		did = str(number)[1:]
	else:
		did = str(number)

	query = sql_signatures.us_jurisdiction_prefix.format(did)
	cur = conn.cursor()
	cur.execute(query)
	for row in cur.fetchall():
		return 'US Local'
	# International
	return 'International'


def get_did_area_code(conn, number):
	# US Toll Free
	if str(number)[0:3] in toll_free:
		return 'US Toll Free'
	if str(number)[1:4] in toll_free:
		return 'US Toll Free'
	# US Local
	query = sql_signatures.us_jurisdiction_prefix.format(number)
	cur = conn.cursor()
	cur.execute(query)
	for row in cur.fetchall():
		print ("get_did_area code US",str(row))
		if row[1] != "": 
		  print ("get_did_code US returned",str(row))
		  return row[0]
	# International
	print("International")
	query = sql_signatures.country_code.format(number, code_deck_id)
	cur = conn.cursor()
	cur.execute(query)
	for row in cur.fetchall():
		print ("get_did_area code EU ",str(row))
		if row[1] != "": 
		  print ("get_did_code EU returned ",str(row))
		  return row[0]
	# Unknown
	print ("Anonymous")
	return "Anonymous"


def get_did_location(conn, number):
	# US Toll Free
	if str(number)[0:3] in toll_free:
		return 'US Toll Free'
	if str(number)[1:4] in toll_free:
		return 'US Toll Free'
	# US Local
	query = sql_signatures.us_jurisdiction_prefix.format(number)
	cur = conn.cursor()
	cur.execute(query)
	for row in cur.fetchall():
		print ("get_did_location US",str(row))
		if row[1] != "": 
		  print ("get_did_location US returned",str(row))
		  return row[1]
	# International
	print("International")
	query = sql_signatures.country_code.format(number, code_deck_id)
	cur = conn.cursor()
	cur.execute(query)
	for row in cur.fetchall():
		print ("get_did_location EU",str(row))
		if row[1] != "": 
		  print ("get_did_location EU returned",str(row))
		  return row[1]
	# Unknown
	print ("Anonymous")
	return "Anonymous"


def get_did_lata(conn, number):
	# US Toll Free
	if str(number)[0:3] in toll_free:
		return 'US Toll Free'
	if str(number)[1:4] in toll_free:
		return 'US Toll Free'
	# US Local
	query = sql_signatures.us_jurisdiction_prefix.format(number)
	cur = conn.cursor()
	cur.execute(query)
	for row in cur.fetchall():
		print ("get_did_lata",str(row))
		if row[2] != "": 
		  print ("get_did_lata returned",str(row))
		  return row[2]
	# International
	return 'OTHERs'


def _authorization_code_summary_report(conn, invoice_dates, egress_ids, auth_token):
	res = []
	if egress_ids:
		headers = {
			"Authorization": "Token {}".format(auth_token)
		}
		params = {
			"start_time": invoice_dates[0],
			"end_time": invoice_dates[1],
			"step": "86400",  #"86400",
			"method": "total",
			"field": "calls",
			"egress_id": egress_ids,
			"group": "dest_number",
			#"output": "http"
		}
		timeout = RT_API_TIMEOUT
		LOG.debug("Start request _authorization_code_summary_report")
		LOG.debug(params)
		calls = requests.get(
			API_HOST, params=params, headers=headers, timeout=timeout)
		LOG.debug("CALLS %s" % str( calls.json())[0:4096] )
		if calls.json()["data"]:
			params["field"] = "egress_billed_time"
			egress_billed_time = requests.get(
				API_HOST, params=params, headers=headers, timeout=timeout)
			LOG.debug("egress_billed_time %s"%str(egress_billed_time))
			params["field"] = "egress_cost"
			egress_cost = requests.get(
				API_HOST, params=params, headers=headers, timeout=timeout)
			d_calls = [item for item in calls.json()["data"] if item]
			d_time = [item for item in egress_billed_time.json()["data"] if item]
			d_cost = [item for item in egress_cost.json()["data"] if item]
			LOG.debug(len(d_calls))
			#LOG.debug(d_time)
			#LOG.debug(d_cost)
			for i, item in enumerate(d_calls):
				if not is_toll_free(item["dest_number"]):
				   LOG.debug("item is not TOLL FREE %s" % str(item) )	  
				   continue
				else:
				   LOG.debug("item is TOLL FREE %s" % str(item) )	  
				#LOG.debug("item %s" % str(item) )
				try:
				 res.append((
					item["dest_number"],
					get_did_type(conn, item["dest_number"]),
					int(item["calls"]),
					round(float(d_time[i]["egress_billed_time"]) / 60, invoice_decimal_digits),
					round(float(d_cost[i]["egress_cost"]), invoice_decimal_digits),
					round(float(item["time"]), invoice_decimal_digits)
				))
				except Exception as e :
				 pass #print(e)
	print ("returned list ",res  )
	return res


def _port_fee_summary_analysis_report(conn, invoice_dates, client_id):
	from_date = datetime.fromtimestamp(float(invoice_dates[0])).strftime('%Y-%m-%d')
	query = sql_signatures.port_fee_summary_analysis.format(invoice_date=from_date, client_id=client_id)
	LOG.debug("port fee_summary_analysis_report with recurring_charges")
	LOG.debug(query)
	result = []
	cur = conn.cursor()
	cur.execute(query)
	charges_list = []
	port_total_charges = 0.0 
	for row in cur.fetchall():
		LOG.debug("Charges list %s" % str(charges_list) )
		if row[0] in charges_list:
		   LOG.debug("This fee already exists %s" % row[0] )
		   continue
		charges_list.append(row[0])
		result.append((
			#datetime.fromtimestamp(float(invoice_dates[1])).strftime('%Y-%m-%d'),
			row[0],
			round(float(row[1]), invoice_decimal_digits),round(float(row[2]), invoice_decimal_digits),round(float(row[3]), invoice_decimal_digits),  )   )
		port_total_charges += round(float(row[3]), invoice_decimal_digits)

	return result,port_total_charges



def _transaction_summary_analysis_report(conn, invoice_dates, client_id):
	from_date = datetime.fromtimestamp(float(invoice_dates[0])).strftime('%Y-%m-%d')
	query = sql_signatures.transaction_summary_analysis.format(invoice_date=from_date, client_id=client_id)
	LOG.debug("transaction_summary_analysis_report with recurring_charges")
	LOG.debug(query)
	result = []
	cur = conn.cursor()
	cur.execute(query)
	charges_list = []
	recurring_charges = 0.0 
	for row in cur.fetchall():
		LOG.debug("Charges list %s" % str(charges_list) )
		if row[0] in charges_list:
		   LOG.debug("This fee already exists %s" % row[0] )
		   continue
		charges_list.append(row[0])
		result.append((
			datetime.fromtimestamp(float(invoice_dates[1])).strftime('%Y-%m-%d'),
			row[0],
			round(float(row[1]), invoice_decimal_digits)))
		if  row[0].find("Monthly Fee") > -1:
			recurring_charges += round(float(row[1]), invoice_decimal_digits)

	return result,recurring_charges


def _get_recurring_charges(conn, client_id, invoice_dates):
	return 0.0


def _get_prev_balance(conn, client_id, invoice_dates):
	'''
	Previous Balance $5.09   --  This is the ending balance of the last day 
	of previous month after the last month’s invoice was sent.
	The table is balance_history_actual
	'''
	balance = 0.0
	from_date = datetime.fromtimestamp(float(invoice_dates[0])).strftime('%Y-%m-%d')
	query = sql_signatures.get_balance.format(client_id, from_date)
	cur = conn.cursor()
	cur.execute(query)
	for row in cur.fetchall():
		balance = row[0]

	return round(float(balance), invoice_decimal_digits)


def _get_token():
	resp = requests.post(AUTH_HOST)
	if not resp.status_code in (200,201,202,204):
		LOG.error('rt_index api auth error')
		raise Exception('rt_index api auth error')
	return resp.json()["token"]


def _get_last_invoice_number(conn):
	query = sql_signatures.last_invoice_number
	cur = conn.cursor()
	cur.execute(query)
	for row in cur.fetchall():
		return row[0]

	return 1


def _check_new_invoice_number(conn, last_invoice_number):
	query = sql_signatures.check_invoice_number.format(last_invoice_number)
	cur = conn.cursor()
	cur.execute(query)
	for row in cur.fetchall():
		return row[0]

	return None


def _get_new_invoice_number(conn):
	last_invoice_number = _get_last_invoice_number(conn)
	while _check_new_invoice_number(conn, last_invoice_number):
		last_invoice_number = _get_last_invoice_number(conn)

	return last_invoice_number






async def _collect_context(conn, client_id, invoice_dates,due,params_options):
	LOG.debug("Start collect context")
	company_name = None
	egress_ids = []
	auth_token = _get_token()
	LOG.info("Received auth token <{}>".format(auth_token))

	query = sql_signatures.inv_requisites
	cur = conn.cursor()
	cur.execute(query)
	for row in cur.fetchall():
		tpl_number = row[0]
		pdf_tpl = row[1]
		company_info = row[2]
		company_info_location = row[3]
		invoice_decimal_digits = row[4]

	query = sql_signatures.client_last_invoiced.format(client_id=client_id)
	cur = conn.cursor()
	cur.execute(query)
	try:
		client_last_invoiced = cur.fetchone()[0]
		print("client_last_invoiced",client_last_invoiced)
		include_taxes = client_last_invoiced.month < datetime.today().month
		print ()
	except :
		traceback.print_exc(file=sys.stdout)
		include_taxes = True
	
	include_taxes = True
	
	##get decimal places
	query = sql_signatures.decimal_place.format(client_id)
	cur = conn.cursor()
	cur.execute(query)
	try:
		invoice_decimal_digits = cur.fetchone()[0]
	except:
		traceback.print_exc(file=sys.stdout)
		invoice_decimal_digits = 2
	
	taxes_list=[] 
	if include_taxes == True:
		query = sql_signatures.include_tax.format(client_id)
		cur = conn.cursor()
		cur.execute(query)
		for row in cur.fetchall():
			taxes_list.append( [row[0],row[1]])


	query = sql_signatures.company_name.format(client_id)
	cur = conn.cursor()
	cur.execute(query)
	for row in cur.fetchall():
		company_name = row[0]
	
	query = sql_signatures.egress_ids.format(client_id)
	cur = conn.cursor()
	cur.execute(query)
	for row in cur.fetchall():
		egress_ids = row[0]
	LOG.debug("Company name <{}>, egress_ids <{}>".format(company_name, egress_ids))
	# ,,,,
	print ("params_options",str(params_options)  )
	try:  
		if params_options['auth_code_800_summary'] == True:
			try:
				authorization_code_summary = _authorization_code_summary_report(
				conn, invoice_dates, egress_ids, auth_token)
			except:
				traceback.print_exc(file=sys.stdout)
				authorization_code_summary =[]
		else:
			authorization_code_summary =[]
			
		#LOG.debug("Start collect aacs")
		if params_options['all_area_codes_summary'] == True:
			try:
				all_area_codes_summary = _all_area_codes_summary_report(
				conn, invoice_dates, egress_ids, auth_token)
			except:
				traceback.print_exc(file=sys.stdout)
				all_area_codes_summary =[]
		else:
			all_area_codes_summary= []
		#LOG.debug("Start collect ols")
		if params_options['orig_lata_summary'] == True:
			try:
				origination_lata_summary = _origination_lata_summary_report(
			conn, invoice_dates, egress_ids, auth_token)
			except:
				traceback.print_exc(file=sys.stdout)
				origination_lata_summary =[]
		else:
			origination_lata_summary= []
		#LOG.debug("Start coleect tsa")
		if  True:   #params_options['transaction_summary_analysis'] ==
			try:
				transaction_summary_analysis,recurring_charges = _transaction_summary_analysis_report(
				conn, invoice_dates, client_id)
			except:
				traceback.print_exc(file=sys.stdout)
				recurring_charges= 0.0
				transaction_summary_analysis= []
		else:
			recurring_charges= 0.0
			transaction_summary_analysis= []
		#print( authorization_code_summary,all_area_codes_summary ,origination_lata_summary ,transaction_summary_analysis )

	except Exception as e:
		traceback.print_exc(file=sys.stdout)
		print ("Exception in Get context", e)
		query = sql_signatures.client_invoice_zero.format(client_id=client_id)
		cur = conn.cursor()
		cur.execute(query)
		try:
			invoice_zero = cur.fetchone()[0] #['invoice_zero']
		except:
			traceback.print_exc(file=sys.stdout)
			invoice_zero = False
		if invoice_zero == True:
			authorization_code_summary =[]
			all_area_codes_summary= []
			origination_lata_summary= []
			recurring_charges= 0.0
			transaction_summary_analysis= []
		else:
			return "No Calls, Invoice zero is False"
			
	if include_taxes :
		_port_fee_summary_analysis,port_fee_total_charges = _port_fee_summary_analysis_report(
		conn, invoice_dates, client_id)		
	else:
		_port_fee_summary_analysis = []
		port_fee_total_charges = 0.0
	#print ("_port_fee_summary_analysis,port_fee_total_charges",_port_fee_summary_analysis,port_fee_total_charges)
	code_usage = {}
	code_usage_date = {}
	
	LOG.debug("authorization_code_summary %s"% str(authorization_code_summary)  )
	for code in authorization_code_summary:
		if code_usage.get(code[0]+  time.strftime('%Y-%m-%d', time.localtime(code[5]) )):
			code_usage[code[0]+  time.strftime('%Y-%m-%d', time.localtime(code[5]) )] = (
				code_usage[code[0]+  time.strftime('%Y-%m-%d', time.localtime(code[5]) )][0] + code[3],
				code_usage[code[0]+  time.strftime('%Y-%m-%d', time.localtime(code[5]) )][1] + code[4],
				"{} minutes of usage to {}".format( round(float(code_usage[code[0]+  time.strftime('%Y-%m-%d', time.localtime(code[5]) )][0] + code[3] ),   invoice_decimal_digits)  
				   , code[0]),code[-1]  )
		else:
			code_usage[code[0]+  time.strftime('%Y-%m-%d', time.localtime(code[5]) )] = (
				code[3], code[4],
				"{} minutes of usage to {}".format(round(float(code[3]), invoice_decimal_digits), code[0]),code[-1]   )
	# authorization_code_summary [('7180000000', 'International', 1, 1.02, 1.02, 1503122760),
	for code in authorization_code_summary:
		if code_usage_date.get(code[0]+  time.strftime('%Y-%m-%d', time.localtime(code[5]) ) ) :
			code_usage_date[code[0] + time.strftime('%Y-%m-%d', time.localtime(code[5]) )  ] = [ code[0],code[1],
				code_usage_date[code[0]+  time.strftime('%Y-%m-%d', time.localtime(code[5]) )  ] [2]   + code[2],
				round(float(code_usage_date[code[0]+  time.strftime('%Y-%m-%d', time.localtime(code[5]) ) ] [3]   + code[3]) , invoice_decimal_digits ),
				code_usage_date[code[0]+  time.strftime('%Y-%m-%d', time.localtime(code[5]) ) ] [4]   + code[4] ,
				code[5] ]

		else:
			code_usage_date[code[0]+time.strftime('%Y-%m-%d', time.localtime(code[5]) )] = [ code[0],code[1],code[2],code[3], code[4], code[5] ]				
				
	
	

	
	#long_distance_charges_withoutTSA = 0
	if include_taxes :
		try:
			month_charges = sum([item[2] for item in transaction_summary_analysis])
		except:
			month_charges =0.0
	else:
		month_charges =0.0
	
	try:
		long_distance_charges= sum([item[4] for item in all_area_codes_summary])
	except:
		long_distance_charges= 0.0
	print ("monthly_charges",month_charges,"long_distance_charges",long_distance_charges)
	for k, v in code_usage.items():
		long_distance_charges_withoutTSA += round(float(v[1]), invoice_decimal_digits)  #is this used in the moment ?
		transaction_summary_analysis.append((
			#datetime.fromtimestamp(float(invoice_dates[1])).strftime('%Y-%m-%d'),
			datetime.fromtimestamp(v[-1]).strftime('%Y-%m-%d'),
			v[2],
			round(float(v[1]), invoice_decimal_digits)))
	authorization_code_summary = []
	for k, v in code_usage_date.items():
		authorization_code_summary.append( v )
	LOG.debug("authorization_code_summary2 %s"% str(authorization_code_summary) )

	prev_balance = _get_prev_balance(conn, client_id, invoice_dates)
	#try:
	#	long_distance_charges = long_distance_charges_withoutTSA  #sum([item[2] for item in transaction_summary_analysis])
	#except IndexError:
	#	print("INDEXEEROR in long_distance_charges_withoutTSA ") 
	#	long_distance_charges = 0.0
	#if long_distance_charges_withoutTSA  == 0:
	#  print("long_distance_charges_withoutTSA:",long_distance_charges_withoutTSA)
	  #long_distance_charges = 0.0
	  #month_charges = 0 
	#recurring_charges = _get_recurring_charges(conn, client_id, invoice_dates)
	#recurring_charges += month_charges
	
	amount_due = round(long_distance_charges + month_charges, invoice_decimal_digits)  # - recurring_charges
	try:
	 print ("long_distance_charges %s   - recurring_charges2: %s  + month_charges %s" %( long_distance_charges ,recurring_charges,month_charges))
	except Exception as e:
	 print (e)
	try: 
		int(due)
	except:
		due = 0 
	due_date = datetime.now() + dt.timedelta(days=int(due))
	
	calculated_taxes_list = []
	old_amount_due = amount_due
	whole_tax = 0.0
	if include_taxes :
		for  tax_row in  taxes_list:
			tax_percent_num = tax_row [ 1 ]  #percent value of the tax
			tax = round((old_amount_due+port_fee_total_charges)* (float(tax_percent_num)/100), invoice_decimal_digits)
			calculated_taxes_list.append( [tax_row[0],tax]  )
			amount_due += tax
			whole_tax += tax

	#print(tax_percent_num)
	
	amount_due += port_fee_total_charges
	print("port_fee_summary_analysis", _port_fee_summary_analysis)
	
	sql_balance = """INSERT INTO client_balance_operation_action  (client_id, balance,ingress_balance ,update_result,action ,create_by)   VALUES  
	({client_id},{fee},{fee},0,3,'DID Invoice'  )  """
	
	#removing tax from client balance
	#sql_balance = sql_balance.format(client_id = client_id, fee =whole_tax    )
	#print("removing tax from client balance", sql_balance)
	#cur = conn.cursor()
	#cur.execute(sql_balance)
	
	return {
		"company_info": company_info,
		"include_tax":  include_taxes,
		"tax" : calculated_taxes_list    ,        #round(float(tax), invoice_decimal_digits),
		"whole_tax":whole_tax,
		"port_fee_total": round(float(port_fee_total_charges), invoice_decimal_digits),
		"port fee summary report": _port_fee_summary_analysis,
		"pdf_tpl": pdf_tpl,
		"invoice_name": _get_new_invoice_number(conn),
		"invoice_decimal_digits": invoice_decimal_digits,
		"company_info_location": company_info_location,
		"tpl_number": tpl_number,
		"acc_num": "{}_{}".format(client_id, invoice_dates[1]),
		"bill_date": datetime.now().strftime('%m-%d-%Y'),
		"due_date":  due_date.strftime('%m-%d-%Y'),
		"billing_period": "{} - {}".format(
			datetime.fromtimestamp(float(invoice_dates[0])).strftime('%m-%d-%Y'),
			datetime.fromtimestamp(float(invoice_dates[1])).strftime('%m-%d-%Y')),
		"amount_due": round(float(amount_due), invoice_decimal_digits),
		"client_name": company_name,
		"prev_balance": round(float(prev_balance), invoice_decimal_digits),
		"long_distance_charges": round(float(long_distance_charges), invoice_decimal_digits),
		"recurring_charges": round(float(recurring_charges), invoice_decimal_digits),
		"transaction_summary_analysis": transaction_summary_analysis,
		"authorization_code_summary_report": authorization_code_summary,
		"all_area_codes_summary_report": all_area_codes_summary,
		"origination_lata_summary_report": origination_lata_summary,
		"params":params_options
	}


def _ip_address_auth(request):
	peername = request.transport.get_extra_info('peername')
	if peername is not None:
		host, port = peername
		if allowed_hosts[0] == '*' or host in allowed_hosts:
			return True
	return False


async def _render_invoice(conn, client_id, date_end,date_start, filename, create_type,request,due,params_options):
	end_date = datetime.fromtimestamp(float(date_end))
	if date_start  == None:
	  start_date = end_date.replace(day=1, hour=0, minute=0, second=0)
	else:
	  start_date = datetime.fromtimestamp(float(date_start))
	invoice_dates = (
		int(start_date.timestamp()),  int(end_date.timestamp()))
	context = await _collect_context(conn, client_id, invoice_dates,due,params_options)

	LOG.debug("Collected context {}".format(context))
	LOG.debug("filename {}".format(filename))
	if context["client_name"] is None:
		LOG.debug("No client with this client_id")
		return None
	try:
		invoice = Invoice(context, filename)
	except Exception as e:
		traceback.print_exc(file=sys.stdout)
		return None
	if invoice.generate():
		query = sql_signatures.create_invoice.format(
			invoice_number=context["invoice_name"],
			state=1,
			client_id=client_id,
			invoice_start=start_date .timestamp(),  #end_date.replace(day=1, hour=0, minute=0, second=0).timestamp(),
			invoice_end=end_date.timestamp(),
			total_amount=context["amount_due"],
			pdf_path=filename,
			create_type = create_type
			# current_balance=context["prev_balance"]
		)
		LOG.debug("log in DB %s"% query )
		cur = conn.cursor()
		cur.execute(query)
		conn.commit()
		
		query = sql_signatures.get_did_billing_id.format(client_id=client_id)
		LOG.debug("get did_billing id %s"% query )
		cur = conn.cursor()
		cur.execute(query)
		result = cur.fetchone()
		#if len(result) > 0:
		#  did_billing_id = result[0]
		#  query = sql_signatures.zero_did_billing_did_price.format ( did_billing_id = did_billing_id	 )
		#  LOG.debug("zero did_billing price %s"% query )
		#  cur.execute(query)
		#  conn.commit()
		query = sql_signatures.update_last_invoiced.format(
			client_id=client_id)
		LOG.debug("Updating client last invoiced field %s"% query )
		cur.execute(query)
		conn.commit()  
		#print (result) 
		
		
		with open(filename, 'rb') as fd:
			data = fd.read()
		return data
	else:
		LOG.debug("Invoice is not generated")
		return None


@catch_all
async def getPdf(request):
	invoice_id = request.match_info['invoice_id']
	LOG.debug("Invoice file requested {}".format(invoice_id))
	conn = request.app['db']
	query = sql_signatures.get_invoice_path.format(invoice_id=invoice_id)
	LOG.debug("Get invoice path {}".format(query))
	cur = conn.cursor()
	cur.execute(query)
	result = cur.fetchone()
	if len(result) > 0:
		invoice_path = result[0]
		invoice_number = result[1]
	else:
		invoice_path = None

	if  invoice_path != None:
		with open(invoice_path, 'rb') as fd:
			data = fd.read()
		resp = StreamResponse(
				status=200,
				headers=MultiDict({
					'Content-Disposition': 'Attachment;filename={}.pdf'.format(invoice_number),
					'Content-Type': 'application/pdf',
					'Content-Length': str(len(data))}))
		await resp.prepare(request)
		resp.write(data)
		return resp
	else:
		return api_response({'msg': 'Data not found'}, 404)

@catch_all
async def get(request):
	from  get_config import invoice_log_config as config_log
	from logging.handlers import TimedRotatingFileHandler
	from logging.handlers import RotatingFileHandler
	
	if int(config_log.get('by_time',0) )== 1:
		handler = TimedRotatingFileHandler(config_log.get ("invoice_manual_path","invoice_auto.log"),when=config_log.get ("invoice_manual_when",'m'),
		interval= int( config_log.get ("invoice_manual_interval",0) ),backupCount= int( config_log.get ("invoice_manual_backupCount",5)  ) )
	else: # by size
		handler = RotatingFileHandler(config_log.get ("invoice_manual_path","invoice_manual.log"),maxBytes=int( config_log.get ("invoice_manual_maxBytes", 1024*1024*128 ) ),
		backupCount= int( config_log.get ("invoice_manual_backupCount",5)  ) )
	
	if int (config_log.get ("debug_level",0) ) == 1: 
		LOG.setLevel(logging.DEBUG)
	else:
		LOG.setLevel(logging.INFO)
	formatstr = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
	formatstr = '%(asctime)s: %(levelname)s:  %(message)s'
	formatter = logging.Formatter(formatstr)
	handler.setFormatter(formatter)
	LOG.addHandler(handler)
	params_options = dict() #what symmaries to be included
	
	LOG.debug("start new invoice")
	if _ip_address_auth(request):
		try:
			params = request.match_info['params']
			LOG.debug("params {}".format(params))
			try:
				params.index("account_summary")
				params_options['account_summary']= True
			except:
				params_options['account_summary']= False
			try:
				params.index("transaction_summary_analysis")
				params_options['transaction_summary_analysis']= True
			except:
				params_options['transaction_summary_analysis']= False
			try:
				params.index("auth_code_800_summary")
				params_options['auth_code_800_summary']= True
			except:
				params_options['auth_code_800_summary']= False
			try:
				params.index("all_area_codes_summary")
				params_options['all_area_codes_summary']= True
			except:
				params_options['all_area_codes_summary']= False
			try:
				params.index("orig_lata_summary")
				params_options['orig_lata_summary']= True
			except:
				params_options['orig_lata_summary']= False
				
		except Exception as e :
			traceback.print_exc(file=sys.stdout)
			print("setting all params as True")
			params_options['account_summary']= True
			params_options['transaction_summary_analysis']= True
			params_options['authorization_code']= True
			params_options['all_area_codes_summary']= True
			params_options['orig_lata_summary']= True
			params_options['auth_code_800_summary']= True
		client_id = request.match_info['client_id']
		date_end = request.match_info['end']
		try:
		  date_start = request.match_info['start']
		except:
		  date_start = None
		try:
		  create_type = request.match_info['type']
		except:
		  create_type = 1
		try:
		  due = request.match_info['due']
		except:
		  due = 0
		try:
		  monthly = request.match_info['month']
		except:
		  monthly = 1
		LOG.debug("Start date {}, End date  {}".format(date_start,date_end))
		if client_id and date_end:
			conn = request.app['db']
			
			invoices_path = os.path.dirname(os.path.realpath(__file__))
			invoices_path += "/static/files"
			filename = '{}/{}_{}.pdf'.format(invoices_path, client_id, date_end)
			try:
			 data = await _render_invoice(conn, client_id, date_end,date_start, filename, create_type,request,due,params_options)
			except Exception as e:
			 print (e)
			 traceback.print_exc(file=sys.stdout)
			 
			 return api_response({'msg': 'Error in generating new invoice %s'  %str(e) }, 400)
			if data is None:
				return api_response({'msg': 'Data not found'}, 404)
			resp = StreamResponse(
					status=200,
					headers=MultiDict({
						'Content-Disposition': 'Attachment;filename={}_{}.pdf'.format(client_id, date_end),
						'Content-Type': 'application/pdf',
						'Content-Length': str(len(data))}))
			await resp.prepare(request)
			resp.write(data)
			return resp
		else:
			return api_response({'msg': 'Missing parameters'}, 400)
	else:
		return api_response({'msg': 'Forbidden'}, 403)
